//
//  THImageBrowserContentLayer.m
//  ArtGallery
//
//  Created by Scott Stevenson on 11/27/07.
//  Released under a BSD-style license. See License.txt
//

#import "THImageBrowserContentLayer.h"
#import "NSImage-Extras.h"

@interface THImageBrowserContentLayer ()
// main image layer
@property CALayer* imageLayer;
// black-and-white version of layer
@property CALayer* imageLayerAlternate;
// text layer for label
@property CATextLayer* textLayer;
@end

@interface THImageBrowserContentLayer (PrivateUtilities)
// create an image layer
+ (CALayer*)newPrivateImageLayerForImage:(NSImage*)inputImage;
// create a text layer
+ (CATextLayer*)newPrivateTextLayerWithString:(NSString*)inputString;
@end


@implementation THImageBrowserContentLayer

// main image layer
@synthesize imageLayer=_imageLayer;
// black-and-white version of layer
@synthesize imageLayerAlternate=_imageLayerAlternate;
// text layer for label
@synthesize textLayer=_textLayer;
// main content image
@dynamic contentImage;
// title below image
@dynamic title;
// sets whether to display the primary or alternate image
@synthesize shouldDisplayMonochromeImage=_shouldDisplayMonochromeImage;



// create a new layer with the target content size and image
+ (id)layerWithContentSize:(CGSize)baseContentSize andImage:(NSImage*)image
{
    // basic geometry for layer
    CGFloat baseWidth = baseContentSize.width;
    CGFloat baseHeight = baseContentSize.height;
    CGFloat midX = floor ( baseContentSize.width * 0.5 );

    // make as wide as base content, plus extra vertical
    // space for text layer.
    THImageBrowserContentLayer* contentLayer = [[[self class] alloc] init];
    contentLayer.name             = @"contentLayer";    
    contentLayer.bounds           = CGRectMake  ( 0, 0, baseWidth, baseHeight+45 );
    contentLayer.anchorPoint      = CGPointMake ( 0, 1 );

    // get image layer to store actual image content.
    // bounds is determined by image size.
    CALayer* imageLayer = [self newPrivateImageLayerForImage:image];
    imageLayer.delegate = contentLayer;
    imageLayer.position = CGPointMake( CGRectGetMidX( contentLayer.bounds ), baseHeight+45 );    
    contentLayer.imageLayer = imageLayer;
    [contentLayer addSublayer:imageLayer];

    // an "alternate" black-and-white version of the image
    NSImage* alternateImage = [image imageByConvertingToBlackAndWhite];
    CALayer* imageLayerAlt = [self newPrivateImageLayerForImage:alternateImage];
    imageLayerAlt.delegate = contentLayer;
    imageLayerAlt.position = CGPointMake( CGRectGetMidX( contentLayer.bounds ), baseHeight+45 );    
    contentLayer.imageLayerAlternate = imageLayerAlt;

    // make the text layer as wide as the container and
    // 30 units of vertical space.
	CATextLayer* textLayer = [self newPrivateTextLayerWithString:@"Untitled"];
    textLayer.bounds = CGRectMake( 0, 0, baseWidth, 30 );
    textLayer.position = CGPointMake( midX, 0 );
    contentLayer.textLayer = textLayer;
    [contentLayer addSublayer:textLayer];

    return contentLayer;
}


// main content image
- (CGImageRef)contentImage
{
    return (CGImageRef)self.imageLayer.contents;
}


// title below image
- (NSString*)title
{
    return self.textLayer.string;
}

- (void)setTitle:(NSString*)newTitle
{
    self.textLayer.string = newTitle;
}


// sets whether to display the primary or alternate image

- (void)setShouldDisplayMonochromeImage:(BOOL)shouldUseAlternate
{
    _shouldDisplayMonochromeImage = shouldUseAlternate;
    
    // we'll simply swap which layer is currently being used
    // as the image layer. this will likely be animated.
    
    if ( shouldUseAlternate ) {
        [self replaceSublayer:self.imageLayer with:self.imageLayerAlternate];
     } else {
        [self replaceSublayer:self.imageLayerAlternate with:self.imageLayer];
    }
}


// reverses the current state of shouldDisplayMonochromeImage

- (void)toggleMonochromeState
{
    self.shouldDisplayMonochromeImage = !(self.shouldDisplayMonochromeImage);
}


// sent by superlayer when mouse is clicked within this layer's frame

- (void)mouseDownAtPointInSuperlayer:(CGPoint)inputPoint
{
    CGPoint point = [self convertPoint:inputPoint fromLayer:self.superlayer];

    // if the image is clicked, play a sound effect and flip to the monochrome state.
    if ( CGRectContainsPoint ( self.imageLayer.frame, point ) )
    {
        [[NSSound soundNamed:@"WaterDroplet"] play];
        [self toggleMonochromeState];
    }
        
}



#pragma mark -
#pragma mark CALayer Subclass Overrides

// returns a custom animation. return [super actionForKey:] to
// get the default animation.

- (id<CAAction>)actionForKey:(NSString *)key
{        
    if ( [key isEqualToString:@"sublayers"] )
    {
        // this animation is triggered when the image layer is swapped
        // with its "alternate" black-and-white version.
        //
        // we'll use the current event to figure out where
        // to center the droplet and to see if the shift key
        // was held down for a slow-motion effect.
        NSEvent* event = [NSApp currentEvent];
        
        // create a ripple transition using the CIRippleTransition filter
        CATransition *rippleTransition = [CATransition animation];
        CIFilter *rippleFilter = [CIFilter filterWithName:@"CIRippleTransition"];
        [rippleFilter setDefaults];
        
        // image will be used for "environment map" of ripple
        NSString* rippleImagePath = [[NSBundle mainBundle] pathForImageResource:@"Ripple"];
        CIImage* rippleImage = [CIImage imageWithData:[NSData dataWithContentsOfFile:rippleImagePath]];
        
        // setup geometry for effect
        NSPoint location = [[NSApp currentEvent] locationInWindow];
        CGSize imageFrameSize = self.imageLayer.frame.size;
        CGFloat width  = imageFrameSize.width;
        CGFloat height = imageFrameSize.height;        
        CIVector* centerVector = [CIVector vectorWithX:location.x Y:location.y];
        CIVector* extentVector = [CIVector vectorWithX:0.0 Y:0.0 Z:width W:height];
        
        // apply input to filter
        [rippleFilter setValue:rippleImage  forKey:kCIInputShadingImageKey];
        [rippleFilter setValue:centerVector forKey:kCIInputCenterKey];
        [rippleFilter setValue:extentVector forKey:kCIInputExtentKey];
        [rippleFilter setValue:[NSNumber numberWithFloat:5.0] forKey:kCIInputScaleKey];

        // apply effect in slow motion if shift key is held down
        BOOL useSlowMotion = NO;
        if ( [event modifierFlags] & NSShiftKeyMask )
            useSlowMotion = YES;
        
        if ( useSlowMotion )        
            rippleTransition.duration = 2.0;
        else
            rippleTransition.duration = 1.0;

        // attach filter to transition and return
        rippleTransition.filter = rippleFilter;            
        return rippleTransition;
    }
    
    // for all other cases, use the default action.
    return [super actionForKey:key];
}



#pragma mark -
#pragma mark Private Utilities


+ (CALayer*) newPrivateImageLayerForImage:(NSImage*)inputImage
{
    // convert to CGImageRef so we can easily set it as layer contents
    CGImageRef cgInputImage = [inputImage cgImage];
    NSSize imageSize = inputImage.size;
    
    // border drawn around image
    CGColorRef borderColor = CGColorCreateGenericGray ( 1.0, 1.0 );
    
    // center image in container layer and set contents to
    // resize while maintaining proportions (aspect ratio).
    // add a 60% shadow with 3 unit offset and a 4 unit border.
    CALayer* imageLayer = [CALayer layer];    
    imageLayer.name                 = @"imageLayer";
    imageLayer.contents             = (id)cgInputImage;
    imageLayer.bounds               = CGRectMake( 0, 0, imageSize.width, imageSize.height );
    imageLayer.anchorPoint          = CGPointMake( 0.5, 1.0 );
    imageLayer.contentsGravity      = kCAGravityResizeAspect;
    imageLayer.shadowOpacity        = 0.6;
    imageLayer.shadowOffset         = CGSizeMake( 3, -3 );
    imageLayer.borderColor          = borderColor;
    imageLayer.borderWidth          = 4.0;

    // cleanup
    CGImageRelease ( cgInputImage );
    CGColorRelease ( borderColor );

    return imageLayer;
}


+ (CATextLayer*)newPrivateTextLayerWithString:(NSString*)inputString;
{
    // create text layer for image title. text is truncated, if
    // necessary. bold 20 unit font by default. unlike image layer,
    // the anchorpoint is 0 for the y axis, meaning the coordinates
    // start in the bottom so we can easily position it from the
    // bottom of the superlayer.
	CATextLayer* textLayer = [CATextLayer layer];
    textLayer.name                  = @"textLayer";
    textLayer.anchorPoint           = CGPointMake( 0.5, 0.0 );
    textLayer.string                = inputString;
    textLayer.font                  = [NSFont boldSystemFontOfSize:20.0];
    textLayer.fontSize              = 20.0;
    textLayer.shadowOffset          = CGSizeMake ( 1,-1 );
	textLayer.shadowOpacity         = 1.0;
    textLayer.alignmentMode         = kCAAlignmentCenter;
    textLayer.truncationMode        = kCATruncationEnd;
    
    return textLayer;
}


@end
