//
//  THImageBrowserView.m
//  ArtGallery
//
//  Created by Scott Stevenson on 11/20/07.
//  Released under a BSD-style license. See License.txt
//

#import "THImageBrowserView.h"
#import <QuartzCore/QuartzCore.h>
#import "THImageBrowserContentLayer.h"
#import "NSImage-Extras.h"



#pragma mark -
#pragma mark Private Declarations


@interface THImageBrowserView ()
// content
@property CALayer* contentContainerLayer;
@property THImageBrowserContentLayer* previousContentLayer;
@property THImageBrowserContentLayer* currentContentLayer;
@property THImageBrowserContentLayer* nextContentLayer;
// navigation
@property CGRect previousNavRect;
@property CGRect nextNavRect;
@property CALayer* navControlContainerLayer;
@property CALayer* navControlLeftLayer;
@property CALayer* navControlRightLayer;
// visual styling
@property CALayer* leftShadow;
@property CALayer* rightShadow;
// tracks the type for the current navigation event.
// NSLeftMouseDown or NSKeyDown.
@property NSEventType navigationEventType;
@end

@interface THImageBrowserView (LayoutPrivate)
- (CGPoint)pointForPreviousOffscreenContentLayer;
- (CGPoint)pointForPreviousContentLayer;
- (CGPoint)pointForCurrentContentLayer;
- (CGPoint)pointForNextContentLayer;
- (CGPoint)pointForNextOffscreenContentLayer;
@end

@interface THImageBrowserView (LayerSetupPrivate)
- (void)setupLayers;
- (void)setupNavigationLayers;
- (void)setupVisualStylingLayers;
- (void)setupTrackingAreas;
@end

@interface THImageBrowserView (NavigationPrivate)
// variations on the public navigation UI methods. In these cases,
// we want to show and hide the arrows based on key presses, not
// whether content is available. see -moveToNextImage: and
// -moveToPreviousImage: for details.
- (void)showLeftArrowForKeyboardEvent;
- (void)showRightArrowForKeyboardEvent;
- (void)hideNavigationArrows;
@end

@interface THImageBrowserView (UtilitiesPrivate)
// create a new instance of THImageBrowserContentLayer for index in imageDataDictionaries
- (THImageBrowserContentLayer*)layerForImageAtIndex:(NSUInteger)index;
// loads image data from source into imageDataDictionaries
- (void)loadImageData;
@end



#pragma mark -
#pragma mark Main Class Implementation

@implementation THImageBrowserView

@synthesize imageDataDictionaries=_imageDataDictionaries;
@synthesize currentImageIndex=_currentImageIndex;
@synthesize baseContentSize=_baseContentSize;
@synthesize contentContainerLayer=_contentContainerLayer;
@synthesize previousContentLayer=_previousContentLayer;
@synthesize currentContentLayer=_currentContentLayer;
@synthesize nextContentLayer=_nextContentLayer;
@synthesize visibleMarginForNextImage=_visibleMarginForNextImage;
@synthesize previousNavRect=_previousNavRect;
@synthesize nextNavRect=_nextNavRect;
@synthesize navControlContainerLayer=_navControlContainerLayer;
@synthesize navControlLeftLayer=_navControlLeftLayer;
@synthesize navControlRightLayer=_navControlRightLayer;
@synthesize leftShadow=_leftShadow;
@synthesize rightShadow=_rightShadow;
@synthesize navigationEventType=_navigationEventType;


- (id)initWithFrame:(NSRect)frame
{
    if ( self = [super initWithFrame:frame] )
    {
        
    }
    return self;
}


- (void)awakeFromNib
{
    // get the view frame as a basis for the layer frame, but
    // make sure the y axis is zero.
    CGRect viewFrame = NSRectToCGRect( self.frame );
    viewFrame.origin.y = 0;

    // create a layer and match its frame to the view's frame
    self.wantsLayer = YES;
    CALayer* mainLayer = self.layer;
    mainLayer.name = @"mainLayer";
    mainLayer.frame = viewFrame;
    mainLayer.delegate = self;
    
    // causes the layer content to be drawn in -drawRect:
    [mainLayer setNeedsDisplay];
    
    // create the basic layout geometry
    self.baseContentSize = CGSizeMake ( 450, 350 );
    self.visibleMarginForNextImage = 25.0;
    
    // setup image content, tracking areas and layers
    [self loadImageData];
    [self setupTrackingAreas];
    [self setupLayers];
    [self setupNavigationLayers];
    [self setupVisualStylingLayers];
}


- (void)drawRect:(NSRect)rect
{
    // draw a basic gradient for the view background
    
    NSColor* gradientBottom = [NSColor colorWithCalibratedWhite:0.10 alpha:1.0];
    NSColor* gradientTop    = [NSColor colorWithCalibratedWhite:0.35 alpha:1.0];
    
	NSGradient* gradient = [[NSGradient alloc] initWithStartingColor:gradientBottom
                                                         endingColor:gradientTop];
    [gradient drawInRect:self.bounds angle:90.0];
}




#pragma mark -
#pragma mark Sublayers (Private)

// the position for each of these layers is automatically adjusted
// by the setter. we're just overriding the setters for these
// properties. the compiler will generate the getters.

- (void)setPreviousContentLayer:(THImageBrowserContentLayer*)newLayer
{
    _previousContentLayer = newLayer;
    newLayer.position = [self pointForPreviousContentLayer];
}

- (void)setCurrentContentLayer:(THImageBrowserContentLayer*)newLayer
{
    _currentContentLayer = newLayer;
    newLayer.position = [self pointForCurrentContentLayer];
}

- (void)setNextContentLayer:(THImageBrowserContentLayer*)newLayer
{
    _nextContentLayer = newLayer;
    newLayer.position = [self pointForNextContentLayer];
}



#pragma mark -
#pragma mark Mouse Events

- (void)mouseDown:(NSEvent*)anEvent
{
    // get the location of the mouse click
    NSPoint location = [self convertPoint:[anEvent locationInWindow] fromView:nil];    
    CGPoint cgLocation = NSPointToCGPoint(location);
    
    // if it hit one of the clickable navigation areas, activate the action method    
    if ( CGRectContainsPoint ( self.previousNavRect, cgLocation ))
    {
        self.navigationEventType = NSLeftMouseDown;
        [self moveToPreviousImage:self];
        return;
    }
    
    if ( CGRectContainsPoint ( self.nextNavRect, cgLocation ))
    {
        self.navigationEventType = NSLeftMouseDown;
        [self moveToNextImage:self];
        return;
    }
    
    // the mouse was clicked in the current image, pass a mouse click
    // off to that layer.
    if ( CGRectContainsPoint ( self.currentContentLayer.frame, cgLocation ))
    {
        [self.currentContentLayer mouseDownAtPointInSuperlayer:cgLocation];
        return;
    }
}

- (BOOL)acceptsFirstResponder
{
    // we need to be first responder in order to accept keyboard events
    return YES;
}

- (void)keyDown:(NSEvent*)event
{
    // we don't want to allow the key to be held down
    if ( event.isARepeat ) return;
    
    // get the key for this event
    NSString* allKeysAsString = [event charactersIgnoringModifiers];
    unichar key = 0;
    if ( allKeysAsString.length > 0 )
        key = [allKeysAsString characterAtIndex:0];
    
    // left and right keys can advance slides
    if ( key == NSLeftArrowFunctionKey )
    {
        self.navigationEventType = NSKeyDown;
        [self moveToPreviousImage:self];
    } else if ( key == NSRightArrowFunctionKey ) {
        self.navigationEventType = NSKeyDown;
        [self moveToNextImage:self];
    }        
}

- (void)keyUp:(NSEvent*)event
{
    // once the key is up, hide the arrows
    [self hideNavigationArrows];
}

- (void)mouseEntered:(NSEvent*)anEvent
{
    // when the mouse enters one of the key tracking areas,
    // display the navigation UI.
    CGRect eventRect = NSRectToCGRect ( anEvent.trackingArea.rect );
    
    if ( CGRectEqualToRect ( eventRect, self.previousNavRect ) ||
         CGRectEqualToRect ( eventRect, self.nextNavRect ) )
     {
         [self showNavigation];
     }
}

- (void)mouseExited:(NSEvent*)anEvent
{
    // hide all navigation when the mouse exits the tracking area
    CGRect eventRect = NSRectToCGRect ( anEvent.trackingArea.rect );
    
    if ( CGRectEqualToRect ( eventRect, self.previousNavRect ) ||
         CGRectEqualToRect ( eventRect, self.nextNavRect ) )
     {
         [self hideNavigation];
     }
}

- (BOOL)mouseDownCanMoveWindow
{
   return NO;
}



#pragma mark -
#pragma mark Action Methods


- (IBAction) moveToPreviousImage:(id)anArgument
{
    if ( self.previousContentLayer == nil ) return;
    NSSound* slideSound = [NSSound soundNamed:@"SlideAdvance"];
    slideSound.volume = 0.05;
    [slideSound play];
    
    // if the shift key is held down, do the crowd-favorite slow-motion effect
    BOOL useSlowMotion = NO;
    if ( [[NSApp currentEvent] modifierFlags] & NSShiftKeyMask )
        useSlowMotion = YES;

    // the index of the image which is currently displayed
    // in the center of the view.
    NSUInteger currentImageIndex = self.currentImageIndex;
    
    // if there are at least two images before the current one, load
    // an upcoming image into the queue.    
    THImageBrowserContentLayer* newContentLayer = nil;
    NSInteger indexToLoad = (currentImageIndex - 2);
    
    if ( indexToLoad >= 0 )
    {
        [CATransaction begin];

            // create the new image layer and position it offscreen to
            // slide into view in the next section.
            newContentLayer = [self layerForImageAtIndex:indexToLoad];
            newContentLayer.position = [self pointForPreviousOffscreenContentLayer];
            [self.contentContainerLayer addSublayer:newContentLayer];

        [CATransaction commit];
    }
    
    // now we'll actually move all the images to the right by
    // adjusting the position of their content layers, including
    // the one we may have just created above.
    [CATransaction begin];
    
        if ( useSlowMotion ) {
            [CATransaction setValue:[NSNumber numberWithFloat:1.5]
                             forKey:kCATransactionAnimationDuration];
        }
    
		// unload the layer which will not longer be visible. save the "alternate"
		// state in the backing dictionary data store. saving state could be
		// fancier, but works fine for this case.
		THImageBrowserContentLayer* nextLayer = self.nextContentLayer;
        if ( nextLayer )
        {
            [nextLayer removeFromSuperlayer];
            BOOL useAlternate = nextLayer.shouldDisplayMonochromeImage;
            NSUInteger indexForNext = (currentImageIndex+1);        
            NSDictionary* imageDict = [self.imageDataDictionaries objectAtIndex:indexForNext];
            [imageDict setValue:[NSNumber numberWithBool:useAlternate] forKey:@"shouldUseAlternateImage"];            
        }
        
        // setting the layers as these instance variables
        // implicitly updates their position so each move
        // one slot to the right.
        self.nextContentLayer      = self.currentContentLayer;        
        self.currentContentLayer   = self.previousContentLayer;
        self.previousContentLayer  = newContentLayer;

        // a subtle difference in UI behavior for mouse and
        // keyboard navigation.
        //
        // if the mouse is being used for navigation, the arrows
        // should indicate if next/previous content is available.
        // if the keyboard is being used, the on-screen arrow for
        // the corresponding arrow on the keyboard should be
        // displayed, but only momentarily. this is somewhat
        // based on the behavior of Spaces.
        
        if ( self.navigationEventType == NSLeftMouseDown ) {            
            [self refreshNavigationState];
        } else if ( self.navigationEventType == NSKeyDown ) {
            [self showLeftArrowForKeyboardEvent];
        }

    [CATransaction commit];
    
    
    // update the current index
	self.currentImageIndex = (currentImageIndex - 1);    
}


- (IBAction) moveToNextImage:(id)anArgument
{    
    if ( self.nextContentLayer == nil ) return;
    NSSound* slideSound = [NSSound soundNamed:@"SlideAdvance"];
    slideSound.volume = 0.05;
    [slideSound play];
    
    // if the shift key is held down, do the crowd-favorite slow-motion effect
    BOOL useSlowMotion = NO;
    if ( [[NSApp currentEvent] modifierFlags] & NSShiftKeyMask )
        useSlowMotion = YES;

    // the index of the image which is currently displayed
    // in the center of the view.
    NSUInteger currentImageIndex = self.currentImageIndex;

    // if there are at least two images after the current one, load
    // an upcoming image into the queue.
    THImageBrowserContentLayer* newContentLayer = nil;              
    NSUInteger indexToLoad = (currentImageIndex + 2);
    NSUInteger maxIndex = (self.imageDataDictionaries.count - 1);  
    
    if ( indexToLoad <= maxIndex )
    {
        [CATransaction begin];
        
            newContentLayer = [self layerForImageAtIndex:indexToLoad];
            newContentLayer.position = [self pointForNextOffscreenContentLayer];
            [self.contentContainerLayer addSublayer:newContentLayer];
            
        [CATransaction commit];
    }
    
    // now we'll actually move all the images to the left by
    // adjusting the position of their content layers, including
    // the one we may have just created above.
    [CATransaction begin];
    
        if ( useSlowMotion ) {
            [CATransaction setValue:[NSNumber numberWithFloat:1.5]
                             forKey:kCATransactionAnimationDuration];
        }

		// unload the layer which will not longer be visible. save the "alternate"
		// state in the backing dictionary data store. saving state could be
		// fancier, but works fine for this case.
        THImageBrowserContentLayer* previousLayer = self.previousContentLayer;
        if ( previousLayer )
        {
            [previousLayer removeFromSuperlayer];
            BOOL useAlternate = previousLayer.shouldDisplayMonochromeImage;
            NSUInteger indexForPrevious = (currentImageIndex-1);        
            NSDictionary* imageDict = [self.imageDataDictionaries objectAtIndex:indexForPrevious];
            [imageDict setValue:[NSNumber numberWithBool:useAlternate] forKey:@"shouldUseAlternateImage"];            
        }

        // setting the layers as these instance variables
        // implicitly updates their position so each move
        // one slot to the left.
        self.previousContentLayer  = self.currentContentLayer;
        self.currentContentLayer   = self.nextContentLayer;
        self.nextContentLayer      = newContentLayer;
        
        // a subtle difference in UI behavior for mouse and
        // keyboard navigation.
        //
        // if the mouse is being used for navigation, the arrows
        // should indicate if next/previous content is available.
        // if the keyboard is being used, the on-screen arrow for
        // the corresponding arrow on the keyboard should be
        // displayed, but only momentarily. this is somewhat
        // based on the behavior of Spaces.

        if ( self.navigationEventType == NSLeftMouseDown ) {            
            [self refreshNavigationState];
        } else if ( self.navigationEventType == NSKeyDown ) {
            [self showRightArrowForKeyboardEvent];
        }

    [CATransaction commit];

    
    // update the current index
	self.currentImageIndex = (currentImageIndex + 1);
}



#pragma mark -
#pragma mark Navigation

- (void)showNavigation
{
    [CATransaction begin];
    
        // there are clickable nav areas on both the left and right sides
        // of the view. when -showNavigation is called, we display the
        // navigation button and hide the layer which does the "darkening"
        // visual effect. the transition is a simple fade by default, so
        // the net effect is that the area is illuminated.
        // 
        // we only do this if navigation is possible in the given direction.
        // that is, if next/previous is not nil.
    
        if ( self.previousContentLayer ) {
			self.navControlLeftLayer.hidden = NO;
			self.leftShadow.hidden = YES;
		} else {
			self.navControlLeftLayer.hidden = YES;
			self.leftShadow.hidden = NO;		    
		}
		
        if ( self.nextContentLayer ) {
			self.navControlRightLayer.hidden = NO;
			self.rightShadow.hidden = YES;
		} else {
			self.navControlRightLayer.hidden = YES;
			self.rightShadow.hidden = NO;		    
		}

    [CATransaction commit];
}

- (void)hideNavigation
{
    [CATransaction begin];

        // hide both navigation layers and display the layers
        // which cause the "darkening" effect.
        
        self.navControlLeftLayer.hidden = YES;
        self.navControlRightLayer.hidden = YES;    
		self.leftShadow.hidden = NO;
		self.rightShadow.hidden = NO;

    [CATransaction commit];
}

- (void)refreshNavigationState
{
    [self showNavigation];
}




#pragma mark -
#pragma mark CALayer Delegate Methods

- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)cgContext;
{    
        
    if ( [layer.name isEqualToString:@"edgeShadowLayer"] )
	{        
        // if these is one of the layers meant for the "darkening" effect for
        // either edge of the view, draw a dark gradient in it.

		[NSGraphicsContext saveGraphicsState];

            // get the bounding box to draw in
            NSRect drawingRect = NSRectFromCGRect( CGContextGetClipBoundingBox( cgContext ) );

            // convert the CGContextRef to a NSGraphicsContext for AppKit drawing
            NSGraphicsContext* context = [NSGraphicsContext graphicsContextWithGraphicsPort:cgContext flipped:NO];
		    [NSGraphicsContext setCurrentContext:context];
        
            // setup the gradient
            NSColor* color1 = [NSColor colorWithCalibratedWhite:0.0 alpha:0.6];
            NSColor* color2 = [NSColor colorWithCalibratedWhite:0.0 alpha:0.0];
            NSGradient* gradient = [[NSGradient alloc] initWithStartingColor:color1 endingColor:color2];

            // draw it at the angle specified by the layer. CALayer is a generic KVC container
            CGFloat angle   = [[layer valueForKey:@"gradientAngle"] floatValue];
            [gradient drawInRect:drawingRect angle:angle];
            	                          
    	[NSGraphicsContext restoreGraphicsState];        
		
	} else {
	    
	    // draw all other layers normally
        [super drawLayer:layer inContext:cgContext];
	}
}


- (id<CAAction>) actionForLayer:(CALayer *)layer forKey:(NSString *)event
{
    if (layer == self.contentContainerLayer && [event isEqualToString:@"sublayers"])
    {
        // disable animation for new layers being added to container layer
        return (id<CAAction>)[NSNull null];
    }

    // for everything else, use default animation. note that
    // nil means "default animation" in this case, not "no animation".
    return nil; 
}




#pragma mark -
#pragma mark Layout (Private)

- (CGPoint)pointForPreviousOffscreenContentLayer
{
    // far left, outside of the visible area
    CGPoint point = [self pointForPreviousContentLayer];
    CGSize baseContentSize = self.baseContentSize;
    CGFloat margin = self.visibleMarginForNextImage;

    point.x -= ( baseContentSize.width + margin );
    return point;
}

- (CGPoint)pointForPreviousContentLayer
{
    // left side, slightly visible
    CGRect myFrame = self.layer.frame;
    CGSize baseContentSize = self.baseContentSize;
    CGFloat margin = self.visibleMarginForNextImage;
        
    CGPoint point;
    point.x = floor ( CGRectGetMinX( myFrame ) - (baseContentSize.width - margin) );
    point.y = floor ( CGRectGetMaxY( myFrame ) - 10 );
    return point;
}

- (CGPoint)pointForCurrentContentLayer
{
    // center of view    
    CGRect myFrame = self.layer.frame;
    CGSize baseContentSize = self.baseContentSize;
        
    CGPoint point;
    point.x = floor ( CGRectGetMidX( myFrame ) - floor (baseContentSize.width * 0.5) );
    point.y = floor ( CGRectGetMaxY( myFrame ) - 10 );
    return point;
}

- (CGPoint)pointForNextContentLayer
{
    // right side, slightly visible
    CGRect myFrame = self.layer.frame;
    CGFloat margin = self.visibleMarginForNextImage;
    
    CGPoint point;
    point.x = floor ( CGRectGetMaxX( myFrame ) - margin );
    point.y = floor ( CGRectGetMaxY( myFrame ) - 10 );
    return point;
}

- (CGPoint)pointForNextOffscreenContentLayer
{
    // far right, outside of the visible area
    CGPoint point = [self pointForNextContentLayer];
    CGSize baseContentSize = self.baseContentSize;
    CGFloat margin = self.visibleMarginForNextImage;

    point.x += ( baseContentSize.width - margin );
    return point;
}


#pragma mark -
#pragma mark Layer Setup (Private)

- (void)setupLayers
{
    // get center of main layer
    CALayer* mainLayer = self.layer;    
    CGFloat midX = CGRectGetMidX( mainLayer.frame );
    CGFloat midY = CGRectGetMidY( mainLayer.frame );
    
    // create a "container" layer for all content layers.
    // same frame as the view's master layer, automatically
    // resizes as necessary.    
    CALayer* contentContainer = [CALayer layer];    
    contentContainer.bounds           = mainLayer.bounds;
    contentContainer.delegate         = self;
    contentContainer.anchorPoint      = CGPointMake(0.5,0.5);
    contentContainer.position         = CGPointMake( midX, midY );
    contentContainer.autoresizingMask = (kCALayerWidthSizable|kCALayerHeightSizable);
    [self.layer addSublayer:contentContainer];
    self.contentContainerLayer = contentContainer;    

    
    // setup initial content layers for current and next images    
    THImageBrowserContentLayer* contentLayer;
    NSUInteger count = self.imageDataDictionaries.count;

    [CATransaction begin];
    
        if ( count > 0 ) {
            contentLayer = [self layerForImageAtIndex:0];
            [contentContainer addSublayer:contentLayer];
            self.currentContentLayer = contentLayer;        
        }
    
        if ( count > 1 ) {
            contentLayer = [self layerForImageAtIndex:1];
            [contentContainer addSublayer:contentLayer];
            self.nextContentLayer = contentLayer;        
        }
    
    [CATransaction commit];
}


- (void)setupNavigationLayers
{
    // get the coordinates of the view's main layer
    CGFloat midX = CGRectGetMidX ( self.layer.frame );
    CGFloat midY = CGRectGetMidY ( self.layer.frame );
    CGFloat maxX = CGRectGetMaxX ( self.layer.frame );

    // create a container layer which will hold the individual
    // navigation buttons.
    CALayer* navContainer = [CALayer layer];
    navContainer.delegate = self;
        
    // make the navigation as large as the bounds of the view
    // and center the layer in its superlayer. also autoresize
    // as superlayer resizes.    
    navContainer.bounds             = self.layer.bounds;
    navContainer.anchorPoint        = CGPointMake( 0.5, 0.5 );
    navContainer.position           = CGPointMake( midX, midY );    
    navContainer.autoresizingMask   = (kCALayerWidthSizable|kCALayerHeightSizable);
    
    // add container layer to main view's layer
    [self.layer addSublayer:navContainer];
    self.navControlContainerLayer = navContainer;    
    
    // setup some shared values for the navigation buttons
    CGColorRef bgColor = CGColorCreateGenericRGB( 0.15, 0.15, 0.15, 1.0 );
    CGColorRef borderColor = CGColorCreateGenericRGB( 1, 1, 1, 0.8 );
    CGFloat edgePadding = 8.0;
    CGRect navButtonBounds = CGRectMake( 0, 0, 50, 35 );


    // create left-side nav button using NSImage left arrow template.
    // size to 20x20 and create a CGImageRef version to easily set as layer
    // contents. -cgImage is a custom category method in NSImage-Extras.h
        
    CALayer* navLeft = [CALayer layer];
    navLeft.delegate = self;
    
    NSImage* leftArrowImage = [NSImage imageNamed:NSImageNameLeftFacingTriangleTemplate];
    [leftArrowImage setSize:NSMakeSize(20,20)];
    leftArrowImage = [leftArrowImage imageByFillingVisibleAlphaWithColor:[NSColor whiteColor]];
    CGImageRef cgLeftArrowImage = [leftArrowImage cgImage];

    // setup layer properties. basically, we want a button with a white background
    // and round corners. the image is centered in the frame and the shadow is 60%
    // with 3 pixel offset.
            
    navLeft.name                 = @"navLeft";
    navLeft.opacity              = 0.9;
    navLeft.bounds               = navButtonBounds;
    navLeft.contents             = (id)cgLeftArrowImage;
    navLeft.cornerRadius         = 12.0;
    navLeft.anchorPoint          = CGPointMake( 0.0, 0.5 );
    navLeft.position             = CGPointMake( edgePadding, midY );
    navLeft.backgroundColor      = bgColor;
    navLeft.borderColor          = borderColor;
    navLeft.borderWidth          = 1.5;
    navLeft.contentsGravity      = kCAGravityCenter;
    navLeft.shadowOffset         = CGSizeMake( 3, -3 );
    navLeft.shadowOpacity        = 0.8;
    navLeft.hidden               = YES;

    // save layer as instace variable and add to superlayer
    self.navControlLeftLayer = navLeft;
    [navContainer addSublayer:navLeft];


    // create right-side nav button using NSImage right arrow template.
    // size to 20x20 and create a CGImageRef version to easily set as layer
    // contents. -cgImage is a custom category method in NSImage-Extras.h

    CALayer* navRight = [CALayer layer];
    navRight.delegate = self;
    
    NSImage* rightArrowImage = [NSImage imageNamed:NSImageNameRightFacingTriangleTemplate];
    [rightArrowImage setSize:NSMakeSize(20,20)];
    rightArrowImage = [rightArrowImage imageByFillingVisibleAlphaWithColor:[NSColor whiteColor]];
    CGImageRef cgRightArrowImage = [rightArrowImage cgImage];
    
    // setup layer properties. basically, we want a button with a white background
    // and round corners. the image is centered in the frame and the shadow is 60%
    // with 3 pixel offset.
    
    navRight.name             = @"navRight";
    navRight.opacity          = 0.9;
    navRight.bounds           = navButtonBounds;
    navRight.contents         = (id)cgRightArrowImage;
    navRight.cornerRadius     = 12.0;
    navRight.anchorPoint      = CGPointMake( 1.0, 0.5 );
    navRight.position         = CGPointMake( (maxX - edgePadding), midY );
    navRight.backgroundColor  = bgColor;
    navRight.borderColor      = borderColor;
    navRight.borderWidth      = 1.5;
    navRight.contentsGravity  = kCAGravityCenter;
    navRight.shadowOffset     = CGSizeMake( 3, -3 );
    navRight.shadowOpacity    = 0.9;
    navRight.hidden           = YES;
    
    // save layer as instace variable and add to superlayer
    self.navControlRightLayer = navRight;
    [navContainer addSublayer:navRight];
    
        
    // clean up
    CGImageRelease ( cgLeftArrowImage );
    CGImageRelease ( cgRightArrowImage );
    CGColorRelease ( bgColor );
    CGColorRelease ( borderColor );
}


- (void)setupVisualStylingLayers
{
    // create layers for shadow effect on left and rights sides of view.
    // we don't set contents directly for either layer. instead, the delegate
    // will draw the content in -drawLayer:inContext: when -setNeedsDisplay is
    // sent to the layer. the layer is initially visible, and hidden when the
    // mouse approaches either edge of the view.
    
    
    // layer for left edge    
    CALayer* leftShadow = [CALayer layer];   
    leftShadow.delegate = self;
     
    leftShadow.name             = @"edgeShadowLayer";
    leftShadow.bounds           = self.previousNavRect;
    leftShadow.anchorPoint      = CGPointMake(0,0);
    leftShadow.position         = self.previousNavRect.origin;
    leftShadow.hidden           = NO;
    leftShadow.contentsGravity  = kCAGravityCenter;

    // this will be used by the delegate for drawing the gradient
    [leftShadow setValue:[NSNumber numberWithFloat:0.0] forKey:@"gradientAngle"];
    
    // save layer as an ivar, ask the delegate to draw the layer contents,
    // then add it to the superlayer.
    self.leftShadow = leftShadow;
    [leftShadow setNeedsDisplay];
    [self.navControlContainerLayer addSublayer:leftShadow];


    // layer for right edge    
    CALayer* rightShadow = [CALayer layer];   
    rightShadow.delegate = self;
         
    rightShadow.name             = @"edgeShadowLayer";
    rightShadow.bounds           = self.nextNavRect;
    rightShadow.anchorPoint      = CGPointMake( 0, 0 );
    rightShadow.position         = self.nextNavRect.origin;
    rightShadow.hidden           = NO;
    rightShadow.contentsGravity  = kCAGravityCenter;
        
    // this will be used by the delegate for drawing the gradient
    [rightShadow setValue:[NSNumber numberWithFloat:180.0] forKey:@"gradientAngle"];

    // save layer as an ivar, ask the delegate to draw the layer contents,
    // then add it to the superlayer.
    self.rightShadow = rightShadow;
    [rightShadow setNeedsDisplay];
    [self.navControlContainerLayer addSublayer:rightShadow];    
}


- (void)setupTrackingAreas
{
    // get basic geometry
    CGRect  mainFrame  = self.layer.frame;
    CGFloat mainHeight = mainFrame.size.height;
    CGFloat mainWidth  = mainFrame.size.width;
    CGFloat trackingWidth = 120.0;
    
    // define the "hot" areas for the tracking rects
    NSRect previousNavRect = NSMakeRect( 0, 0, trackingWidth, mainHeight );
    self.previousNavRect   = NSRectToCGRect ( previousNavRect );
    NSRect nextNavRect     = NSMakeRect( ( mainWidth - trackingWidth), 0, trackingWidth, mainHeight);
    self.nextNavRect       = NSRectToCGRect ( nextNavRect );

    // we want notifications for the mouse entering and exiting our
    // tracking areas, and only when the window is the key window.
    NSTrackingArea* trackingArea;
    NSTrackingAreaOptions options = ( NSTrackingMouseEnteredAndExited | NSTrackingActiveInKeyWindow );

    trackingArea = [[NSTrackingArea alloc] initWithRect:previousNavRect
                                                options:options
                                                  owner:self
                                               userInfo:nil];
    [self addTrackingArea:trackingArea];    

    trackingArea = [[NSTrackingArea alloc] initWithRect:nextNavRect
                                                options:options
                                                  owner:self
                                               userInfo:nil];
    [self addTrackingArea:trackingArea];
}



#pragma mark -
#pragma mark Navigation (Private)

// variations on the public navigation UI methods. In these cases,
// we want to show and hide the arrows based on key presses, not
// whether content is available. see -moveToNextImage: and
// -moveToPreviousImage: for details.

- (void)showLeftArrowForKeyboardEvent
{
	self.navControlLeftLayer.hidden = NO;
	self.leftShadow.hidden = YES;
}

- (void)showRightArrowForKeyboardEvent
{
	self.navControlRightLayer.hidden = NO;
	self.rightShadow.hidden = YES;
}

- (void)hideNavigationArrows
{
    [self hideNavigation];
}



#pragma mark -
#pragma mark Utilities (Private)


// create a new instance of THImageBrowserContentLayer for index in imageDataDictionaries

- (THImageBrowserContentLayer*)layerForImageAtIndex:(NSUInteger)index
{
    // fetch source data from imageDataDictionaries array
    NSDictionary* imageDict = [self.imageDataDictionaries objectAtIndex:index];
    NSImage* image = [imageDict valueForKey:@"image"]; 
    NSString* title = [imageDict valueForKey:@"title"];
    BOOL useAlternate = [[imageDict valueForKey:@"shouldUseAlternateImage"] boolValue];
        
    // create a content layer with image dictionary contents
    CGSize size = self.baseContentSize;       
    THImageBrowserContentLayer* contentLayer;
    contentLayer = [THImageBrowserContentLayer layerWithContentSize:size andImage:image];
    contentLayer.title = title;
    
    // this BOOL says if we should display the black-and-white image by default
    if ( useAlternate )
        contentLayer.shouldDisplayMonochromeImage = YES;
                
    return contentLayer;
}


// loads image data from source into imageDataDictionaries

- (void)loadImageData
{
    NSBundle* appBundle = [NSBundle mainBundle];
    NSString* path = [appBundle pathForResource:@"images" ofType:@"plist"];
    NSArray* sourceImagesArray = [[NSDictionary dictionaryWithContentsOfFile:path] valueForKey:@"images"];
    
    // create an array to store the new image dicts we're about to creeate
    NSMutableArray* imageDataDictionaries = [NSMutableArray array];
    
    // loop through all image paths in the plist    
    for ( NSDictionary* imageDict in sourceImagesArray )
    {
        // get image path, create image object, then scale
        // proportionally to target size.        
        NSString* imagePath = [imageDict valueForKey:@"imageName"];
        NSImage* image = [[NSImage alloc] initWithContentsOfFile:imagePath];        

        // get the proporitional size first to get an exact size. we want to
        // get the size first so that we create an image without "padded space"
        // around the edges to fit a certain rectangle. getting an exact size
        // means we get correct centering and avoid adding a border around
        // any empty space.
        NSSize targetSize = NSSizeFromCGSize( self.baseContentSize );        
        NSSize actualSize = [image proportionalSizeForTargetSize:targetSize];        
        image = [image imageByScalingProportionallyToSize:actualSize
                                                  flipped:NO
                                                 addFrame:NO
                                                addShadow:NO
                                                 addSheen:NO];
                                                 
        // get title from dictionary
        NSString* title = [imageDict valueForKey:@"title"];
        
        
        // create new image dictionary with actual real NSImage object
        // and related values.
        NSMutableDictionary* newImageDict = [NSMutableDictionary dictionary];        
        [newImageDict setValue:title forKey:@"title"];
        [newImageDict setValue:image forKey:@"image"];        
        [imageDataDictionaries addObject:newImageDict];        
    }
    
    // set the images array as an ivar and set the initial index
    self.imageDataDictionaries = imageDataDictionaries;
    self.currentImageIndex = 0;    
}

@end